/**
 * \file /input/masschroq/masscgroqmlinfoparser.cpp
 * \date 27/05/2023
 * \author Olivier Langella
 * \brief extracts important informations from masschroqML file
 */

/*******************************************************************************
 * Copyright (c) 2023 Olivier Langella
 *<Olivier.Langella@universite-paris-saclay.fr>.
 *
 * This file is part of i2MassChroQ.
 *
 *     i2MassChroQ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     i2MassChroQ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with i2MassChroQ.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

#include <QDebug>
#include <QDir>
#include <pappsomspp/exception/exceptionnotfound.h>
#include "masschroqmlinfoparser.h"

MasschroqMlInfoParser::MasschroqMlInfoParser()
{
}

MasschroqMlInfoParser::~MasschroqMlInfoParser()
{
}

const QString &
MasschroqMlInfoParser::getTsvResultDirectoryPath() const
{
  return m_tsvdir;
}

const McqrMetadataSet &
MasschroqMlInfoParser::getMcqrMetadataSet() const
{
  return m_mcqrMetadataSet;
}


void
MasschroqMlInfoParser::readStream()
{

  m_isOk = false;
  qDebug();
  if(m_qxmlStreamReader.readNextStartElement())
    {
      qDebug() << m_qxmlStreamReader.name();
      if(m_qxmlStreamReader.name().toString() == "masschroq")
        {
          qDebug();
          while(m_qxmlStreamReader.readNextStartElement())
            {
              qDebug() << m_qxmlStreamReader.name();

              if(m_qxmlStreamReader.name().toString() == "rawdata")
                {
                  read_rawdata();
                }
              else if(m_qxmlStreamReader.name().toString() == "groups")
                {
                  read_groups();
                }
              else if(m_qxmlStreamReader.name().toString() == "protein_list")
                {
                  m_qxmlStreamReader.skipCurrentElement();
                }
              else if(m_qxmlStreamReader.name().toString() == "peptide_list")
                {
                  m_qxmlStreamReader.skipCurrentElement();
                }
              else if(m_qxmlStreamReader.name().toString() == "alignments")
                {
                  m_qxmlStreamReader.skipCurrentElement();
                }
              else if(m_qxmlStreamReader.name().toString() ==
                      "quantification_methods")
                {
                  m_qxmlStreamReader.skipCurrentElement();
                }

              else if(m_qxmlStreamReader.name().toString() == "quantification")
                {
                  read_quantification();
                }
              else
                { // group
                  /*
                m_qxmlStreamReader.raiseError(
                  QObject::tr("unknown element %1")
                    .arg(m_qxmlStreamReader.name()));
                    */
                  m_qxmlStreamReader.skipCurrentElement();
                }
            }
        }
      else
        {
          m_qxmlStreamReader.raiseError(
            QObject::tr("Not a masschroqML file (no masschroq tag)"));
          m_qxmlStreamReader.skipCurrentElement();
        }
    }

  if(errorString().isEmpty())
    {
      m_isOk = true;
    }
  qDebug();
}


bool
MasschroqMlInfoParser::readFile(const QString &fileName)
{
  QDir::setCurrent(QFileInfo(fileName).absolutePath());
  return pappso::XmlStreamReaderInterface::readFile(fileName);
}

void
MasschroqMlInfoParser::read_groups()
{
  /*
  <groups>
      <group id="all_msrun" data_ids="msruna1 msruna2 msruna3 msruna4 msruna5
  msruna6 msruna7 msruna8 msruna9 msrunb10 msrunb11 msrunb12 msrunb13 msrunb14
  msrunb15 msrunb16 msrunb17 msrunb18 msrunb19"/>
  </groups>
  */

  while(m_qxmlStreamReader.readNextStartElement())
    {
      qDebug() << m_qxmlStreamReader.name();

      if(m_qxmlStreamReader.name().toString() == "group")
        {
          QString group_id =
            m_qxmlStreamReader.attributes().value("id").toString();
          QStringList data_id_list = m_qxmlStreamReader.attributes()
                                       .value("data_ids")
                                       .toString()
                                       .split(" ");

          m_mapGroups.insert({group_id, data_id_list});
          m_qxmlStreamReader.skipCurrentElement();
        }
      else
        { // group
          m_qxmlStreamReader.raiseError(
            QObject::tr("unknown element %1").arg(m_qxmlStreamReader.name()));
          m_qxmlStreamReader.skipCurrentElement();
        }
    }
}

void
MasschroqMlInfoParser::read_rawdata()
{

  while(m_qxmlStreamReader.readNextStartElement())
    {
      qDebug() << m_qxmlStreamReader.name();

      if(m_qxmlStreamReader.name().toString() == "data_file")
        {
          //<data_file id="msruna1" format="mzxml"
          // path="/gorgone/pappso/data_extraction_pappso/mzXML/20120906_balliau_extract_1_A01_urnb-1.mzXML"
          // type="centroid"/>
          McqrMetadata metadata;
          metadata.m_msrunId =
            m_qxmlStreamReader.attributes().value("id").toString();

          metadata.m_msrunFile =
            QFileInfo(m_qxmlStreamReader.attributes().value("path").toString())
              .fileName();
          m_mcqrMetadataSet.push_back(metadata);
          m_qxmlStreamReader.skipCurrentElement();
        }
      else
        { // group
          m_qxmlStreamReader.raiseError(
            QObject::tr("unknown element %1").arg(m_qxmlStreamReader.name()));
          m_qxmlStreamReader.skipCurrentElement();
        }
    }
}

void
MasschroqMlInfoParser::read_quantification()
{


  // <quantification>
  /*
<quantification_results>
  <quantification_result output_file = "result_eme" format = "tsv" />
  <quantification_result output_file = "rdata_eme" format = "rdata" />
  <!--<quantification_result output_file = "result2" format = "ods" /> -->
  </ quantification_results><quantification_traces>
  <!--<peptide_traces peptide_ids = "pep0 pep1" output_dir =
         "pep_traces" format      = "tsv" /> --></ quantification_traces>
  <quantify id = "q1" withingroup = "all_msrun" quantification_method_id =
     "quant1">
  <peptides_in_peptide_list mode = "post_matching" ni_min_abundance = "0.8" />
  <!--<mz_list> 732.317 449.754 552.234 464.251 381.577 569.771 575.256 <
   / mz_list> -- > <!--<mzrt_list><mzrt mz = "732.317" rt = "230.712" />
                     <mzrt mz = "575.256" rt = "254.788" /></ mzrt_list> -->
  </ quantify>
*/

  while(m_qxmlStreamReader.readNextStartElement())
    {
      qDebug() << m_qxmlStreamReader.name();

      if(m_qxmlStreamReader.name().toString() == "quantification_results")
        {

          while(m_qxmlStreamReader.readNextStartElement())
            {
              qDebug() << m_qxmlStreamReader.name();

              if(m_qxmlStreamReader.name().toString() ==
                 "quantification_result")
                {
                  //    <quantification_result output_file = "result_eme" format
                  //    = "tsv" />
                  if(m_qxmlStreamReader.attributes()
                       .value("format")
                       .toString() == "tsv")
                    {
                      m_tsvdir = QDir(m_qxmlStreamReader.attributes()
                                        .value("output_file")
                                        .toString())
                                   .absolutePath();
                      m_tsvdir.append(".d");
                      qDebug() << m_tsvdir;
                    }

                  m_qxmlStreamReader.skipCurrentElement();
                }
              else if(m_qxmlStreamReader.name().toString() == "compar_result")
                {
                  m_qxmlStreamReader.skipCurrentElement();
                }
              else if(m_qxmlStreamReader.name().toString() == "peak_shape")
                {
                  m_qxmlStreamReader.skipCurrentElement();
                }
              else if(m_qxmlStreamReader.name().toString() == "xic_coords")
                {
                  m_qxmlStreamReader.skipCurrentElement();
                }

              else

                { // group
                  m_qxmlStreamReader.raiseError(
                    QObject::tr("unknown element %1")
                      .arg(m_qxmlStreamReader.name()));
                  m_qxmlStreamReader.skipCurrentElement();
                }
            }
        }

      else if(m_qxmlStreamReader.name().toString() == "quantify")
        {
          /*
          <quantify id="q1" withingroup="all_msrun"
quantification_method_id="quant1"> <peptides_in_peptide_list
mode="post_matching" ni_min_abundance="0.8"/>
  <!--<mz_list>732.317 449.754 552.234 464.251 381.577 569.771
575.256</mz_list>-->
  <!--<mzrt_list>
<mzrt mz="732.317" rt="230.712" />
<mzrt mz="575.256" rt="254.788" />
</mzrt_list>-->
</quantify>*/

          QString qid = m_qxmlStreamReader.attributes().value("id").toString();
          QString withingroup =
            m_qxmlStreamReader.attributes().value("withingroup").toString();
          qDebug() << qid;
          qDebug() << withingroup;

          m_mapQuantifyIdToGroup.insert({qid, withingroup});

          m_qxmlStreamReader.skipCurrentElement();
        }
      else
        { // group
          /*
        m_qxmlStreamReader.raiseError(
          QObject::tr("unknown element
        %1").arg(m_qxmlStreamReader.name()));
          */
          m_qxmlStreamReader.skipCurrentElement();
        }
    }
  //  </quantification>
}


QString
MasschroqMlInfoParser::getPeptideTsvResultFilename(const QString &q_id) const
{
  // all_msrun_proteins.tsv           peptides_q1_all_msrun.tsv

  auto it_qid = m_mapQuantifyIdToGroup.find(q_id);

  if(it_qid != m_mapQuantifyIdToGroup.end())
    {

      QDir result_dir(m_tsvdir);
      QFileInfo peptide_file(result_dir.absoluteFilePath(
        QString("peptides_%1_%2.tsv").arg(it_qid->first).arg(it_qid->second)));

      if(peptide_file.exists())
        {
          return peptide_file.absoluteFilePath();
        }
      else
        {
          throw pappso::ExceptionNotFound(
            QObject::tr("peptide TSV file %1 not found")
              .arg(peptide_file.absoluteFilePath()));
        }
    }
  else
    {
      throw pappso::ExceptionNotFound(
        QObject::tr("quantification id %1 not found").arg(q_id));
    }
}

QString
MasschroqMlInfoParser::getProteinTsvResultFilename(const QString &q_id) const
{

  QDir result_dir(m_tsvdir);


  auto it_qid = m_mapQuantifyIdToGroup.find(q_id);

  if(it_qid != m_mapQuantifyIdToGroup.end())
    {


      QFileInfo protein_file(result_dir.absoluteFilePath(
        QString("%1_proteins.tsv").arg(it_qid->second)));

      if(protein_file.exists())
        {
          return protein_file.absoluteFilePath();
        }
      else
        {
          throw pappso::ExceptionNotFound(
            QObject::tr("protein TSV file %1 not found")
              .arg(protein_file.absoluteFilePath()));
        }
    }
  else
    {
      throw pappso::ExceptionNotFound(
        QObject::tr("quantification id %1 not found").arg(q_id));
    }
}

const QString &
MasschroqMlInfoParser::getGroupIdbyQid(const QString &q_id) const
{

  auto it_qid = m_mapQuantifyIdToGroup.find(q_id);

  if(it_qid != m_mapQuantifyIdToGroup.end())
    {
      return it_qid->second;
    }
  else
    {
      throw pappso::ExceptionNotFound(
        QObject::tr("quantification id %1 not found").arg(q_id));
    }
}


const QString &
MasschroqMlInfoParser::getSingleQid() const
{
  if(m_mapQuantifyIdToGroup.size() == 0)
    {
      throw pappso::ExceptionNotFound(QObject::tr("no quantification id"));
    }
  if(m_mapQuantifyIdToGroup.size() == 1)
    {
      return m_mapQuantifyIdToGroup.begin()->first;
    }
  else
    {
      throw pappso::ExceptionNotFound(
        QObject::tr("more than one quantification id"));
    }
}

McqrMetadataSet
MasschroqMlInfoParser::getMcqrMetadataSet(const QString &group_id) const
{
  McqrMetadataSet metadata_set;

  auto it_group = m_mapGroups.find(group_id);

  if(it_group != m_mapGroups.end())
    {
      for(auto data_id : it_group->second)
        {
          metadata_set.push_back(
            m_mcqrMetadataSet.getMcqrMetadaByDataId(data_id));
        }
    }
  else
    {
      throw pappso::ExceptionNotFound(
        QObject::tr("group id %1 not found").arg(group_id));
    }
  return metadata_set;
}

bool
MasschroqMlInfoParser::isOk() const
{
  return m_isOk;
}
